#include <cmath>
#include <iostream>
#include <vector>
using namespace std;

class scalarfun_t {
public:
  virtual double eval(double x)=0;
};

class integral_t {
  int N;
public:
  integral_t(int N_a) : N(N_a) {}
  void set_N(int N_a) { N = N_a; }
  double simpson(scalarfun_t &f,double a,double b) {  
    double
      h = (b-a)/N,
      h2 = h/2.0,
      finteg=0.0;
    for (int j=0; j<N; j++) {
      double x = j*h;
      finteg += 2.0*f.eval(x);
      finteg += 4.0*f.eval(x+h2);
    }
    finteg -= f.eval(a);
    finteg += f.eval(b);
    finteg *= h/6.0;
    return finteg;
  }
};

class myfun_t : public scalarfun_t {
public:
  double eval(double x) {
    return sin(x*M_PI);
  }
};

class polyfun_t : public scalarfun_t {
public:
  vector<double> coefs;
  double eval(double x) {
    double px = 0.0;
    int n = coefs.size()-1;
    for (int j=0; j<=n; j++) px = px*x+coefs[j];
    return px;
  }
};

int main() {
  myfun_t f;
  integral_t integ(100);
  cout << "int(sin(pi*x),0,1) [should be " << 2.0/M_PI <<
    "]; " << integ.simpson(f,0,1) << endl;

  polyfun_t p;
  p.coefs.resize(3,0);
  p.coefs[0] = 1;
  cout << "int(x^2,0,1) [should be 1/3]: "
       << integ.simpson(p,0,1) << endl;
  return 0;
}
