#include <cmath>
#include <iostream>
#include <vector>
using namespace std;

class scalarfun_t {
public:
  virtual double eval(double x)=0;
};

class integral1d_t {
public:
  int N;
  double xa,xb;
  scalarfun_t *fp;
  double value() {  
    double
      h = (xb-xa)/N,
      h2 = h/2.0,
      finteg=0.0;
    for (int j=0; j<N; j++) {
      double x = j*h;
      finteg += 2.0*fp->eval(x);
      finteg += 4.0*fp->eval(x+h2);
    }
    finteg -= fp->eval(xa);
    finteg += fp->eval(xb);
    finteg *= h/6.0;
    return finteg;
  }
};

class scalarfun2d_t {
public:
  virtual double eval(double x,double y)=0;
};

class scalarfun_xcnst_t : public scalarfun_t {
public:
  scalarfun2d_t *f2dp;
  double x;
  double eval(double y) { return f2dp->eval(x,y); }
};

class integraly_t : public scalarfun_t{
  integral1d_t int1d;
  scalarfun_xcnst_t fxcnst;
public:
  integraly_t(int N,double ya,double yb,
              scalarfun2d_t &f2d) {
    int1d.N = N;
    int1d.xa = ya;
    int1d.xb = yb;
    int1d.fp = &fxcnst;
    fxcnst.f2dp = &f2d;
  }
  double eval(double x) {
    fxcnst.x = x;
    return int1d.value();
  }
};

class myfun_t : public scalarfun2d_t {
public:
  double eval(double x,double y) {
    // return x*y;
    return sin(M_PI*x)*sin(M_PI*y); // debe dar (2/pi)^2 = 0.40528
  }
} myfun;

int main() {
  int N=100;
  integraly_t integraly(N,0,1,myfun);
#if 0
  for (int j=0; j<=N; j++) {
    double x = double(j)/N;
    cout << "j,x,inty(x): " << j << ", " << x << ", "
         << integraly.eval(x) << endl;
  }
#endif
  integral1d_t integrx;
  integrx.N = N;
  integrx.xa = 0;
  integrx.xb = 1;
  integrx.fp = &integraly;
  cout << "integral(x,y) = " << integrx.value() << endl;
  return 0;
}
