#include <cmath>
#include <iostream>
#include <vector>
using namespace std;

// One variable function 'f(x)'
class scalarfun1d_t {   //L-scfun1d-b
public:
  virtual double eval(double x)=0;
};
//L-scfun1d-e

// Two variables function 'f(x,y)'
class scalarfun2d_t {
public:
  virtual double eval(double x,double y)=0;
};

// Two variables function 'f(x,y,z)'
class scalarfun3d_t {
public:
  virtual double eval(double x,double y,double z)=0;
};

// Integral of a one var funct over an interval
class integral1d_t {
private:
  int N;                        // Nbr of integration pts
  double xa,xb;                 // integration interval
  scalarfun1d_t *fp;              // ptr to function
public:
  // Ctor, set values
  integral1d_t(int Na=0,double xaa=NAN,
               double xba=NAN,scalarfun1d_t *fpa=NULL) {
    set(Na,xaa,xba,fpa);
  }
  // Set values
  void set(int Na,double xaa,double xba,scalarfun1d_t *fpa) {
    N=Na; xa=xaa; xb=xba; fp=fpa;
  }
  // Computes the integral using Simpson's rule
  double integral() {  
    double
      h = (xb-xa)/N,
      h2 = h/2.0,
      finteg=0.0;
    for (int j=0; j<N; j++) {
      double x = j*h;
      double f = fp->eval(x);
      finteg += 2.0*f;
      finteg += 4.0*fp->eval(x+h2);
    }
    finteg -= fp->eval(xa);
    finteg += fp->eval(xb);
    finteg *= h/6.0;
    return finteg;
  }
};

class fy_t : public scalarfun1d_t { //L-int2d-b
public:
  scalarfun2d_t *f2dp;          // Ptr to 2var funct fx,y)
  double y;
  double eval(double x) { return f2dp->eval(x,y); }
} fy;

class g_t : public scalarfun1d_t {
public:
  integral1d_t int1dx;  // integral over x
  double eval(double y) { // wrapper pass to calling class
    fy.y = y;
    return int1dx.integral();
  }
} g;

class integral2d_t {
private:
  int N;                        // Nbr of integr segments
  double xa,xb,ya,yb;           // rectangle corners
  integral1d_t int1dy;          // integral over y
public:
  void set(int Na=0,double xaa=NAN,double xba=NAN,
           double yaa=NAN,double yba=NAN,
           scalarfun2d_t *f2dpa=NULL) {
    N = Na; xa=xaa; xb=xba; ya=yaa; yb=yba;
    fy.f2dp = f2dpa;
    int1dy.set(N,ya,yb,&g);
    g.int1dx.set(N,xa,xb,&fy);
  }
  integral2d_t(int Na=0,          // Ctor
               double xaa=NAN,double xba=NAN,
               double yaa=NAN,double yba=NAN,
               scalarfun2d_t *f2dpa=NULL) {
    set(Na,xaa,xba,yaa,yba,f2dpa);
  }
  double integral() {
    // 2D integral. Integrates over y the
    // integral over x
    return int1dy.integral();
  }
};

// User code starts here----
// Specific 2var function to be integrated
class fun_t : public scalarfun2d_t {
public:
  double eval(double x,double y) {
    // Must give (2/pi)^2 = 0.40528
    return sin(0.5*M_PI*x)*sin(0.5*M_PI*y); 
  }
} fun;

int main() {
  int N=10;

  // 2D integration example
  integral2d_t int2d(N,0.0,1.0,0.0,1.0,&fun);
  cout << "integral(x,y) = " << int2d.integral() << endl;

  return 0;
}
//L-int2d-2
